@@ -19,8 +19,8 @@
  *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifndef __LINUX_DETECT_H__
-#define __LINUX_DETECT_H__
+#ifndef __BSD_DETECT_H__
+#define __BSD_DETECT_H__
 
 #include <stdio.h>
 
