@@ -36,8 +36,5 @@
 #elif defined(HAVE_LIBUSB_1_0)
 # include "linux/link_usb1.c"
 #endif
-#ifdef HAVE_LINUX_TICABLE_H
-# include "linux/link_dev.c"
-#endif
 
 #endif
