@@ -24,7 +24,7 @@
 
 int log_hex_start(void);
 int log_hex_1(int dir, uint8_t data);
-int log_hex_N(int dir, uint8_t *data, int len);
+int log_hex_N(int dir, const uint8_t *data, uint32_t len);
 int log_hex_stop(void);
 
 #endif
